package io.github.wulkanowy.ui.modules.settings.mod_settings

import android.content.SharedPreferences
import android.os.Bundle
import android.view.View
import androidx.appcompat.app.AlertDialog
import androidx.preference.EditTextPreference
import androidx.preference.MultiSelectListPreference
import androidx.preference.PreferenceFragmentCompat
import com.google.android.material.dialog.MaterialAlertDialogBuilder
import dagger.hilt.android.AndroidEntryPoint
import io.github.wulkanowy.R
import io.github.wulkanowy.data.repositories.PreferencesRepository
import io.github.wulkanowy.ui.base.BaseActivity
import io.github.wulkanowy.ui.base.ErrorDialog
import io.github.wulkanowy.ui.modules.dashboard.DashboardItem
import io.github.wulkanowy.ui.modules.main.MainActivity
import io.github.wulkanowy.ui.modules.main.MainView
import io.github.wulkanowy.utils.AppInfo
import timber.log.Timber
import javax.inject.Inject

@AndroidEntryPoint
class ModSettingsFragment : PreferenceFragmentCompat(),
    SharedPreferences.OnSharedPreferenceChangeListener,
    MainView.TitledView, ModSettingsView {

    @Inject
    lateinit var presenter: ModSettingsPresenter

    @Inject
    lateinit var appInfo: AppInfo

    @Inject
    lateinit var preferencesRepository: PreferencesRepository

    override val titleStringId get() = R.string.pref_settings_advanced_title

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)
        presenter.onAttachView(this)
    }

    override fun onCreatePreferences(savedInstanceState: Bundle?, rootKey: String?) {
        setPreferencesFromResource(R.xml.scheme_preferences_mod_settings, rootKey)

        val attendancePercentagePreference: EditTextPreference? = findPreference("attendance_percentage")
        attendancePercentagePreference?.setOnBindEditTextListener { editText ->
            editText.inputType = android.text.InputType.TYPE_CLASS_NUMBER or android.text.InputType.TYPE_NUMBER_FLAG_DECIMAL
            editText.filters = arrayOf(
                android.text.InputFilter { source, _, _, dest, _, _ ->
                    if (source == "." && dest.isEmpty()) {
                        return@InputFilter "0."
                    }

                    val input = dest.toString() + source.toString()
                    if (input == "100.00") {
                        return@InputFilter null
                    }

                    val inputVal = input.toFloatOrNull()
                    if (inputVal != null && inputVal >= 0 && inputVal <= 100) {
                        null
                    } else {
                        ""
                    }
                }
            )
        }
    }

    override fun onSharedPreferenceChanged(sharedPreferences: SharedPreferences?, key: String?) {
        presenter.onSharedPreferenceChanged(key)
    }

    override fun showError(text: String, error: Throwable) {
        (activity as? BaseActivity<*, *>)?.showError(text, error)
    }

    override fun showMessage(text: String) {
        (activity as? BaseActivity<*, *>)?.showMessage(text)
    }

    override fun showExpiredCredentialsDialog() {
        (activity as? BaseActivity<*, *>)?.showExpiredCredentialsDialog()
    }

    override fun onCaptchaVerificationRequired(url: String?) {
        (activity as? BaseActivity<*, *>)?.onCaptchaVerificationRequired(url)
    }

    override fun showDecryptionFailedDialog() {
        (activity as? BaseActivity<*, *>)?.showDecryptionFailedDialog()
    }

    override fun showChangePasswordSnackbar(redirectUrl: String) {
        (activity as? BaseActivity<*, *>)?.showChangePasswordSnackbar(redirectUrl)
    }

    override fun openClearLoginView() {
        (activity as? BaseActivity<*, *>)?.openClearLoginView()
    }

    override fun showErrorDetailsDialog(error: Throwable) {
        ErrorDialog.newInstance(error).show(childFragmentManager, error.toString())
    }

    override fun showAuthDialog() {
        (activity as? BaseActivity<*, *>)?.showAuthDialog()
    }

    override fun onResume() {
        super.onResume()
        preferenceScreen.sharedPreferences?.registerOnSharedPreferenceChangeListener(this)
    }

    override fun onPause() {
        super.onPause()
        preferenceScreen.sharedPreferences?.unregisterOnSharedPreferenceChangeListener(this)
    }

    override fun showAttendanceSettings(items: List<DashboardItem.HiddenAttendanceTile>) {
        val entries = requireContext().resources.getStringArray(R.array.mod_settings_attendance_entries)
        val values = requireContext().resources.getStringArray(R.array.mod_settings_attendance_values)
        val selectedItemsState = values.map { value -> items.any { it.name == value } }

        MaterialAlertDialogBuilder(requireContext())
            .setTitle(R.string.attendance_title)
            .setMultiChoiceItems(entries, selectedItemsState.toBooleanArray()) { _, _, _ -> }
            .setPositiveButton(android.R.string.ok) { dialog, _ ->
                val selectedState = (dialog as AlertDialog).listView.checkedItemPositions
                val selectedValues = values
                    .filterIndexed { index, _ -> selectedState[index] }
                    .map { DashboardItem.HiddenAttendanceTile.valueOf(it) }

                Timber.i("Selected attendance to hide: $selectedValues")
                presenter.onAttendanceSettingsSelected(selectedValues)
            }
            .setNegativeButton(android.R.string.cancel) { _, _ -> }
            .show()
    }

    override fun restartApp() {
        startActivity(MainActivity.getStartIntent(requireContext()))
        requireActivity().finishAffinity()
    }
}
